//  WebNoteHappyAppDelegate.h
//  WebNoteHappy
//
//  Created by Luis de la Rosa on 8/15/05.
//  Copyright Happy Apps LLC 2005 . All rights reserved.

#import <Cocoa/Cocoa.h>
#import <Growl/Growl.h>
#import <WebKit/WebKit.h>
#import "URLWatcherDelegate.h"
#import "FoldersController.h"
#import "TagsController.h"

@class DeliciousFacade;
@class BookmarkServices;

#import "RBSplitView.h"
#import "RBSplitSubview.h"
#import "SpotlightExporter.h"
#import "KNShelfSplitView.h"
#import "PreferencesController.h"
#import "LicenseController.h"
#import "FaviconDownloader.h"
#import "HAWebnote.h"
#import "WebPreviewGenerator.h"
#import "Folder.h"
#import "ManualFolder.h"
#import "DBSourceSplitView.h"
#import "DBListSplitView.h"
#import "PreviewAreaView.h"
#import "LRFilterBar.h"
#import "BrokenOutletChecker.h"
@class AddEditWebnoteWithTagsPanelController;

@class AddEditWebnotePanelController;

extern NSString *HAAddWebnoteHotKeyKeyCode;
extern NSString *HAAddWebnoteHotKeyModifiers;

extern NSString *HASearchWebnotesHotKeyKeyCode;
extern NSString *HASearchWebnotesHotKeyModifiers;

extern NSString *HAAddWebnoteWithoutPanelHotKeyKeyCode;
extern NSString *HAAddWebnoteWithoutPanelHotKeyModifiers;

@interface WebNoteHappyAppDelegate : NSObject <URLWatcherDelegate, GrowlApplicationBridgeDelegate, WebPreviewGeneratorDelegate>
{
    IBOutlet NSWindow *window;
	IBOutlet NSTableView *webNoteTable;
	IBOutlet NSArrayController *webNoteArrayController;
	IBOutlet NSTextField *currentURLTextField;
	IBOutlet NSTextView *noteTextView;
	IBOutlet NSTableColumn *titleTableColumn;
	IBOutlet NSTableColumn *creationDateTableColumn;
	IBOutlet NSSearchField *searchField;
	IBOutlet NSWindow *importDialog;
	IBOutlet NSProgressIndicator *importProgressIndicator;
	IBOutlet NSMenuItem *hideToolbarMenuItem;
	IBOutlet NSWindow *upgradeDialog;
	IBOutlet NSProgressIndicator *upgradeProgressIndicator;
	
	IBOutlet PreferencesController *preferencesController;
	IBOutlet NSPanel *preferencesPanel;
	IBOutlet NSTextField *addWebnoteHotKeyDescriptionField;
	IBOutlet NSTextField *addWebnoteWithoutPanelHotKeyDescriptionField;
	IBOutlet NSTextField *searchWebnotesHotKeyDescriptionField;
	
	IBOutlet NSOutlineView *foldersOutlineView;
	IBOutlet NSTokenField *tagsField;
	IBOutlet NSBrowser *tagsBrowser;
	IBOutlet NSPanel *credentialsPanel;
	
	IBOutlet FoldersController *foldersController;
	
	IBOutlet TagsController *tagsController;
	
	IBOutlet DeliciousFacade *deliciousFacade;
	BookmarkServices *bookmarkServices;
	
	IBOutlet SpotlightExporter *spotlightExporter;
	
	IBOutlet RBSplitView *browserWebnotesAndNoteSplitView;
	IBOutlet RBSplitSubview *tagBrowserSubView;
	IBOutlet NSMenuItem *toggleTagBrowserVisibilityMenuItem;	
	IBOutlet NSPanel *inspector;
	
	IBOutlet NSProgressIndicator *mainProgressIndicator;
	
	IBOutlet LicenseController *licenseController;
	
	IBOutlet NSUserDefaultsController *defaultsController;
		
    NSManagedObjectModel *managedObjectModel;
    NSManagedObjectContext *managedObjectContext;
	
	NSPanel *infoPanel;
	
	BOOL saveOnQuit;
	
	IBOutlet NSMenu *bookmarkContextMenu;
	FaviconDownloader *faviconDownloader;
  
  WebPreviewGenerator *previewGenerator;  
  BOOL previewAreaHidden;
  IBOutlet NSButton *previewAreaToggleButton;
  
	IBOutlet DBSourceSplitView *mainSplitter;
	IBOutlet NSView *sourceViewPlaceholder;
	IBOutlet NSView *contentViewPlaceholder;
	IBOutlet NSView *sourceView;
	IBOutlet NSView *contentView;
  
  IBOutlet DBListSplitView *sourceSplitter;
	IBOutlet NSView *sourceListViewPlaceholder;
	IBOutlet NSView *previewAreaViewPlaceholder;
  IBOutlet NSView *sourceListView;
  IBOutlet PreviewAreaView *previewAreaView;
  
  IBOutlet NSView *sourcebar;
  IBOutlet NSView *statusbar;
  
  IBOutlet LRFilterBar *filterBar;
  
  int splitterFrameHeightOffset;
  
  AddEditWebnoteWithTagsPanelController *addEditWebnoteWithTagsPanelController;
}

- (NSManagedObjectModel *)managedObjectModel;
- (NSManagedObjectContext *)managedObjectContext;

- (IBAction)saveAction:sender;

- (IBAction)importFromSafari:sender;
- (IBAction)importFromLegacyWebNoteHappy:sender;
- (IBAction)clear:sender;

- (IBAction)openInBrowser:sender;
- (IBAction)newWebNoteFromSafari:sender;
- (IBAction)searchWebNotes:sender;
- (IBAction)delete:sender;

- (IBAction)hideToolbar:sender;
- (IBAction)customizeToolbar:sender;
- (IBAction)showMainWindow:sender;
- (IBAction)closeMainWindow:sender;
- (IBAction)installSafariBookmarklet:sender;
- (IBAction)renameSelectedWebnote:(id)sender;
- (IBAction)editSelectedWebnote:(id)sender;

- (IBAction)reportABug:(id)sender;
- (IBAction)sendFeedback:(id)sender;
- (IBAction)visitHappyApps:(id)sender;

- (IBAction)setAddWebnoteHotkey: (id)sender;
- (IBAction)setAddWebnoteWithoutPanelHotkey: (id)sender;
- (IBAction)setSearchWebnotesHotkey: (id)sender;
	
- (IBAction)closeInfoPanel:(id)sender;
- (IBAction)showWordsPanel:(id)sender;

- (IBAction)toggleTagBrowserVisibility:(id)sender;
- (IBAction)showInspector:(id)sender;

- (void)showWebnotesForWords: (NSArray *)words;
- (void)selectWebNote:(NSManagedObject *)webNote;

+ (NSString *)getMetadataForKey:(NSString *)key inStoreWithPath:(NSString *)path inContext:(NSManagedObjectContext *)context;

- (void)createAndSelectWebNoteWithURL:(NSString *)url title:(NSString *)title selectedText:(NSString *)selectedText;

- (HAWebnote *)selectOrCreateWebnoteWithURL:(NSString *)url title:(NSString *)title selectedText:(NSString *)selectedText;

- (BOOL)webNoteExistsForURL:(NSString *)url;
- (HAWebnote *)webNoteForURL:(NSString *)url;

- (void)shareWebnote:(HAWebnote *)webnote;
- (void)unshareWebnote:(HAWebnote *)webnote;

- (void)createWebNotesFromDictionaryArray:(NSArray *)arrayOfDictionaries;
- (void)importFromDelicious:(id)sender;
-(void)deleteWebnoteWithURL:(NSString *)url;

-(IBAction)newFolder:(id)sender;
-(IBAction)newSmartFolder:(id)sender;

-(IBAction)shareSelectedWebnotes:(id)sender;
-(IBAction)saveSelectedWebnotesPrivately:(id)sender;
-(IBAction)unshareSelectedWebnotes:(id)sender;

-(IBAction)shareSelectedWebnotesToDelicious:(id)sender;
-(IBAction)saveSelectedWebnotesPrivatelyToDelicious:(id)sender;
-(IBAction)unshareSelectedWebnotesFromDelicious:(id)sender;

-(IBAction)showPreferencesPanel:(id)sender;
-(IBAction)copyLinkForSelectedWebnotes:(id)sender;

-(IBAction)updateInterfaceSize:(id)sender;
-(void)updateInterfaceToSize:(int)size;
- (NSString *)applicationSupportFolder;
- (void)newFaviconAvailable;

-(BookmarkServices *)bookmarkServices;

- (ManualFolder *)currentManualFolder;
- (void)addURLStringToCurrentManualFolder:(NSString *)urlString;

- (IBAction)togglePreviewArea:(id)sender;

- (IBAction)resetPreviewAndFaviconCaches:(id)sender;

- (void)refreshMainWindow;

@end
